@@ -85,6 +85,7 @@ ds_syntax_dict = {'snapshot': BOOLEAN_REGEX,
         'all_snapshots': BOOLEAN_REGEX,
         'replicate_append_basename': BOOLEAN_REGEX,
         'replicate_append_fullname': BOOLEAN_REGEX,
+        'recursive': BOOLEAN_REGEX,
         'replicate_full_clone': BOOLEAN_REGEX,
         'replicate_receive_save': BOOLEAN_REGEX,
         'replicate_receive_no_mountpoint': BOOLEAN_REGEX,
@@ -97,6 +98,7 @@ ds_syntax_dict = {'snapshot': BOOLEAN_REGEX,
         'replicate_target': ds_name_syntax,
         'replicate_source': ds_name_syntax,
         'replicate_endpoint': NETCMD_REGEX,
+        'replicate_endpoint_login': USER_REGEX,
         'replicate_endpoint_host': HOST_REGEX,
         'replicate_endpoint_port': PORT_REGEX,
         'replicate_endpoint_command': SHELLFORMAT_REGEX,
@@ -112,6 +114,7 @@ ds_syntax_dict = {'snapshot': BOOLEAN_REGEX,
         'replicate2_send_raw': BOOLEAN_REGEX,
         'replicate2_target': ds_name_syntax,
         'replicate2_endpoint': NETCMD_REGEX,
+        'replicate2_endpoint_login': USER_REGEX,
         'replicate2_endpoint_host': HOST_REGEX,
         'replicate2_endpoint_port': PORT_REGEX,
         'replicate2_endpoint_command': SHELLFORMAT_REGEX,
@@ -128,8 +131,9 @@ ds_syntax_dict = {'snapshot': BOOLEAN_REGEX,
         'remote2_clean_all': BOOLEAN_REGEX,
         'template': template_name_syntax,
         }
+DEFAULT_ENDPOINT_LOGIN = 'root'
 DEFAULT_ENDPOINT_PORT = 22
-DEFAULT_ENDPOINT_CMD = 'ssh -p {port} {host}'
+DEFAULT_ENDPOINT_CMD = 'ssh -l {login} -p {port} {host}'
 DATE_SPEC = '%Y%m%d '
 ZFS_MOUNTPOINT_NONE = ('legacy', 'none')
 
@@ -495,15 +499,19 @@ class Config(object):
                         and (ds_config.has_option(dataset, 'replicate_target') or ds_config.has_option(dataset, 'replicate_source'))):
                     host = ds_config.get(dataset, 'replicate_endpoint_host', fallback='')
                     port = ds_config.get(dataset, 'replicate_endpoint_port', fallback=DEFAULT_ENDPOINT_PORT)
+                    login = ds_config.get(dataset, 'replicate_endpoint_login', fallback=DEFAULT_ENDPOINT_LOGIN)
                     if ds_config.has_option(dataset, 'replicate_endpoint_host'):
                         command = ds_config.get(dataset, 'replicate_endpoint_command', fallback=DEFAULT_ENDPOINT_CMD)
                         if host:
-                            endpoint = command.format(port=port, host=host)
+                            endpoint = command.format(port=port, host=host, login=login)
                         else:
                             endpoint = ''
                     else:
                         endpoint = ds_config.get(dataset, 'replicate_endpoint')
+                    recursive = ds_config.getboolean(dataset, 'recursive', fallback=False)
                     full_clone = ds_config.getboolean(dataset, 'replicate_full_clone', fallback=False)
+                    if recursive:
+                        full_clone = True
                     send_properties = ds_config.getboolean(dataset, 'replicate_send_properties', fallback=False)
                     append_basename, append_fullname = check_ds_config_clash('replicate_append_basename', 'replicate_append_fullname')
                     receive_no_mountpoint, receive_mountpoint = check_ds_config_clash('replicate_receive_no_mountpoint', 'replicate_receive_mountpoint', 
@@ -529,6 +537,7 @@ class Config(object):
                                                       'all_snapshots': ds_config.getboolean(dataset, 'all_snapshots',
                                                             fallback=old_setting_repl_all),
                                                       'compression': ds_config.get(dataset, 'compression', fallback=None),
+                                                      'recursive': recursive,
                                                       'full_clone': full_clone,
                                                       'receive_save': ds_config.getboolean(dataset, 'replicate_receive_save', fallback=False),
                                                       'receive_no_mountpoint': receive_no_mountpoint,
@@ -547,15 +556,19 @@ class Config(object):
                         and (ds_config.has_option(dataset, 'replicate2_target'))):
                     host = ds_config.get(dataset, 'replicate2_endpoint_host', fallback='')
                     port = ds_config.get(dataset, 'replicate2_endpoint_port', fallback=DEFAULT_ENDPOINT_PORT)
+                    login = ds_config.get(dataset, 'replicate2_endpoint_login', fallback=DEFAULT_ENDPOINT_LOGIN)
                     if ds_config.has_option(dataset, 'replicate2_endpoint_host'):
                         command = ds_config.get(dataset, 'replicate2_endpoint_command', fallback=ds_config.get(dataset, 'replicate_endpoint_command', fallback=DEFAULT_ENDPOINT_CMD))
                         if host:
-                            endpoint = command.format(port=port, host=host)
+                            endpoint = command.format(port=port, host=host, login=login)
                         else:
                             endpoint = ''
                     else:
                         endpoint = ds_config.get(dataset, 'replicate2_endpoint')
+                    recursive = ds_config.getboolean(dataset, 'recursive', fallback=False)
                     full_clone = ds_config.getboolean(dataset, 'replicate2_full_clone', fallback=False)
+                    if recursive:
+                        full_clone = True
                     send_properties = ds_config.getboolean(dataset, 'replicate2_send_properties', fallback=False)
                     append_basename, append_fullname = check_ds_config_clash('replicate2_append_basename', 'replicate2_append_fullname')
                     receive_no_mountpoint, receive_mountpoint = check_ds_config_clash('replicate2_receive_no_mountpoint', 'replicate2_receive_mountpoint', 
@@ -581,6 +594,7 @@ class Config(object):
                                                       'all_snapshots': ds_config.getboolean(dataset, 'all_snapshots',
                                                             fallback=old_setting_repl_all),
                                                       'compression': ds_config.get(dataset, 'compression2', fallback=None),
+                                                      'recursive': recursive,
                                                       'full_clone': full_clone,
                                                       'append_basename': ds_config.get(dataset, 'replicate2_append_basename', fallback=False),
                                                       'append_fullname': ds_config.get(dataset, 'replicate2_append_fullname', fallback=False),
