@@ -178,14 +178,14 @@ class Manager(object):
         return result
 
     @staticmethod
-    def snapshot(dataset, snapshots, now, local_dataset='', endpoint='', log_command=False):
+    def snapshot(dataset, snapshots, now, local_dataset='', endpoint='', log_command=False, recursive=False):
         local_dataset = dataset if not local_dataset else local_dataset
         result = PROC_EXECUTED
         this_time = time.strftime(SNAPSHOTNAME_FMTSPEC, time.localtime(now))
         # Take this_time's snapshotzfs
         log_info('[{0}] - Taking snapshot {1}@{2}'.format(local_dataset, dataset, this_time))
         try:
-            ZFS.snapshot(dataset, this_time, endpoint=endpoint, log_command=log_command)
+            ZFS.snapshot(dataset, this_time, endpoint=endpoint, log_command=log_command, recursive=recursive)
         except Exception as ex:
             # if snapshot fails move onto next one
             log_error('[{0}] -   Exception: {1}'.format(local_dataset, str(ex)))
@@ -197,14 +197,14 @@ class Manager(object):
         return result
 
     @staticmethod
-    def new_hold(dataset, snap_name, endpoint='', log_command=False):
+    def new_hold(dataset, snap_name, endpoint='', log_command=False, recursive=False):
         result = PROC_EXECUTED
         holds = ZFS.holds(dataset, endpoint=endpoint, log_command=log_command)
-        ZFS.hold(dataset, snap_name, endpoint=endpoint, log_command=log_command, may_exist=True)
+        ZFS.hold(dataset, snap_name, endpoint=endpoint, log_command=log_command, may_exist=True, recursive=recursive)
         if snap_name in holds:
             holds.remove(snap_name)
         for hold in holds:
-            ZFS.release(dataset, hold, endpoint=endpoint, log_command=log_command)
+            ZFS.release(dataset, hold, endpoint=endpoint, log_command=log_command, recursive=recursive)
         result = PROC_CHANGED
         return result
 
@@ -218,6 +218,7 @@ class Manager(object):
         src_host = gethostname().split('.')[0] if push else replicate_settings['endpoint_host']
         dst_host = gethostname().split('.')[0] if not push else replicate_settings['endpoint_host']
         local_dataset = src_dataset if push else dst_dataset
+        recursive = replicate_settings['recursive']
         full_clone = replicate_settings['full_clone']
         receive_save = replicate_settings['receive_save']
         receive_no_mountpoint = replicate_settings['receive_no_mountpoint']
@@ -257,8 +258,8 @@ class Manager(object):
                     all_snapshots=all_snapshots)
             snapshot = list(new_dst_snapshots)[-1]
             snap_name = new_dst_snapshots[snapshot]['name']
-            Manager.new_hold(src_dataset, snap_name, endpoint=src_endpoint, log_command=log_command)
-            Manager.new_hold(dst_dataset, snap_name, endpoint=dst_endpoint, log_command=log_command)
+            Manager.new_hold(src_dataset, snap_name, endpoint=src_endpoint, log_command=log_command, recursive=recursive)
+            Manager.new_hold(dst_dataset, snap_name, endpoint=dst_endpoint, log_command=log_command, recursive=recursive)
             dst_snapshots.clear()
             dst_snapshots.update(new_dst_snapshots)
             log_info('[{0}] - Resumed replicatiion [{1}]:{2} to [{3}]:{4} complete'.format(local_dataset, src_host, src_dataset, dst_host, dst_dataset))
@@ -287,8 +288,8 @@ class Manager(object):
                 log_info('[{0}] -   {1}@{2} > {1}@{3} ({4})'.format(local_dataset, src_dataset, prevsnap_name, snap_name, size))
                 ZFS.replicate(src_dataset, prevsnap_name, snap_name, dst_dataset, replicate_settings['endpoint'],
                         direction=replicate_dirN, **extra_args)
-                Manager.new_hold(src_dataset, snap_name, endpoint=src_endpoint, log_command=log_command)
-                Manager.new_hold(dst_dataset, snap_name, endpoint=dst_endpoint, log_command=log_command)
+                Manager.new_hold(src_dataset, snap_name, endpoint=src_endpoint, log_command=log_command, recursive=recursive)
+                Manager.new_hold(dst_dataset, snap_name, endpoint=dst_endpoint, log_command=log_command, recursive=recursive)
                 for snapshot in snaps_to_send:
                     dst_snapshots.update({snapshot:src_snapshots[snapshot]})
                 result = PROC_CHANGED
@@ -301,8 +302,8 @@ class Manager(object):
                     log_info('[{0}] -   {1}@{2} > {1}@{3} ({4})'.format(local_dataset, src_dataset, prevsnap_name, snap_name, size))
                     ZFS.replicate(src_dataset, prevsnap_name, snap_name, dst_dataset, replicate_settings['endpoint'],
                             direction=replicate_dirN, **extra_args)
-                    Manager.new_hold(src_dataset, snap_name, endpoint=src_endpoint, log_command=log_command)
-                    Manager.new_hold(dst_dataset, snap_name, endpoint=dst_endpoint, log_command=log_command)
+                    Manager.new_hold(src_dataset, snap_name, endpoint=src_endpoint, log_command=log_command, recursive=recursive)
+                    Manager.new_hold(dst_dataset, snap_name, endpoint=dst_endpoint, log_command=log_command, recursive=recursive)
                     previous_snapshot = snapshot
                     dst_snapshots.update({snapshot:src_snapshots[snapshot]})
                     result = PROC_CHANGED
@@ -314,8 +315,8 @@ class Manager(object):
             log_info('  {0}@         > {0}@{1} ({2})'.format(src_dataset, snap_name, size))
             ZFS.replicate(src_dataset, None, snap_name, dst_dataset, replicate_settings['endpoint'],
                     direction=replicate_dirN, **extra_args)
-            Manager.new_hold(src_dataset, snap_name, endpoint=src_endpoint, log_command=log_command)
-            ZFS.hold(dst_dataset, snap_name, endpoint=dst_endpoint, log_command=log_command)
+            Manager.new_hold(src_dataset, snap_name, endpoint=src_endpoint, log_command=log_command, recursive=recursive)
+            ZFS.hold(dst_dataset, snap_name, endpoint=dst_endpoint, log_command=log_command, recursive=recursive)
             if full_clone:
                 for snapshot in src_snapshots:
                     dst_snapshots.update({snapshot:src_snapshots[snapshot]})
@@ -358,6 +359,8 @@ class Manager(object):
                 replicate2_settings = dataset_settings['replicate2']
                 full_clone = replicate_settings['full_clone'] if replicate else False
                 full_clone2 = replicate2_settings['full_clone'] if replicate2 else False
+                recursive = replicate_settings['recursive'] if replicate else False
+                recursive2 = replicate2_settings['recursive'] if replicate2 else False
                 log_command = dataset_settings['log_commands']
                 local_snapshots = snapshots.get(dataset, OrderedDict())
                 # Manage what snapshots we operate on - everything or zsnapd only
@@ -380,11 +383,11 @@ class Manager(object):
 
                     result = PROC_FAILURE
                     if (take_snapshot is True and this_time not in local_snapshots):
-                        result = Manager.snapshot(dataset, local_snapshots, now, log_command=log_command)
+                        result = Manager.snapshot(dataset, local_snapshots, now, log_command=log_command, recursive=recursive)
                     # Clean snapshots if one has been taken - clean will not execute
                     # if no snapshot taken
                     Cleaner.clean(dataset, local_snapshots, dataset_settings['schema'], log_command=log_command,
-                            all_snapshots=dataset_settings['clean_all'])
+                            all_snapshots=dataset_settings['clean_all'], recursive=recursive)
                     # Execute postexec command
                     if result and dataset_settings['postexec'] is not None:
                             Helper.run_command(dataset_settings['postexec'], '/', log_command=log_command)
@@ -407,7 +410,7 @@ class Manager(object):
                         # Clean snapshots remotely if one has been taken - only kept snapshots will allow aging
                         if (dataset_settings['remote_schema']):
                             Cleaner.clean(remote_dataset, remote_snapshots, dataset_settings['remote_schema'], log_command=log_command,
-                                    all_snapshots=dataset_settings['remote_clean_all'])
+                                    all_snapshots=dataset_settings['remote_clean_all'], recursive=recursive)
                     if (replicate2 is True):
                         # If network replicating, check connectivity here
                         test_unconnected = is_connected.test_unconnected(replicate2_settings, local_dataset=dataset)
@@ -423,7 +426,7 @@ class Manager(object):
                         # Clean snapshots remotely if one has been taken - only kept snapshots will allow aging
                         if (dataset_settings['remote2_schema']):
                             Cleaner.clean(remote_dataset, remote_snapshots, dataset_settings['remote2_schema'], log_command=log_command,
-                                    all_snapshots=dataset_settings['remote2_clean_all'])
+                                    all_snapshots=dataset_settings['remote2_clean_all'], recursive=recursive2)
                     # Post execution command
                     if ((result or result2) and dataset_settings['replicate_postexec'] is not None):
                         Helper.run_command(dataset_settings['replicate_postexec'], '/', log_command=log_command)
@@ -438,7 +441,7 @@ class Manager(object):
                         continue
                     
                     remote_dataset = replicate_settings['target'] if push else replicate_settings['source']
-                    remote_datasets = ZFS.get_datasets(replicate_settings['endpoint'], log_command=log_command)
+                    remote_datasets = ZFS.get_datasets(replicate_settings['endpoint'], remote_dataset, log_command=log_command)
                     if remote_dataset not in remote_datasets:
                         log_error("[{0}] - remote dataset '{1}' does not exist".format(dataset, remote_dataset))
                         continue
@@ -454,10 +457,11 @@ class Manager(object):
 
                         # Take remote snapshot
                         result = PROC_FAILURE
-                        result = Manager.snapshot(remote_dataset, remote_snapshots, now, endpoint=endpoint, local_dataset=dataset, log_command=log_command)
+                        result = Manager.snapshot(remote_dataset, remote_snapshots, now, endpoint=endpoint, local_dataset=dataset, log_command=log_command,
+                                                  recursive=recursive)
                         # Clean remote snapshots if one has been taken - only kept snapshots will aging to happen
                         Cleaner.clean(remote_dataset, remote_snapshots, dataset_settings['schema'], log_command=log_command,
-                                endpoint=endpoint, local_dataset=dataset, all_snapshots=dataset_settings['clean_all'])
+                                endpoint=endpoint, local_dataset=dataset, all_snapshots=dataset_settings['clean_all'], recursive=recursive)
                         # Execute remote postexec command
                         if result and dataset_settings['postexec'] is not None:
                                 Helper.run_command(dataset_settings['postexec'], '/', endpoint=endpoint, log_command=log_command)
@@ -468,7 +472,7 @@ class Manager(object):
                         # Clean snapshots locally if one has been taken - only kept snapshots will allow aging
                         #if not replicate_settings['full_clone']:
                         Cleaner.clean(dataset, local_snapshots, dataset_settings['local_schema'], log_command=log_command,
-                                all_snapshots=dataset_settings['local_clean_all'])
+                                all_snapshots=dataset_settings['local_clean_all'], recursive=recursive)
                         # Post execution command
                         if (result and dataset_settings['replicate_postexec'] is not None):
                             Helper.run_command(dataset_settings['replicate_postexec'], '/', endpoint=endpoint, log_command=log_command)
